


// Includes definitions relevant to the peripherals embedded in the digital signal controller.
#include "F2806x_Device.h"

#include "constants.h"

#include "hardware_interface.h"




// Definition of external functions. The bodies of the functions are in another file
extern int Initialize_the_Clocks(void);
extern int Initialize_the_GPIO(void);
extern int Initialize_the_Timer0(int);
extern int Initialize_the_ADC(void);
extern int Initialize_the_PWM(void);
extern int Initialize_the_Encoder(void);
extern int Enable_the_Peripherals(void);
extern int Manage_LEDs(int, int);
extern int Show_Character_on_Digit(char,int);
extern int Sample_the_Analog_Voltage(void);
extern int Sample_the_Temperature(void);
extern int Set_PWM_Duty_Cycles(int, int, int);

extern int Enable_the_Interrupts(void);


extern const float Sine_table[128];
extern float Interpolate_Sine_Table(float, float*);

int Number_to_Char_Converter(int,char*);

// Global variables are defined outside any function.
int Contatore=0;

int Global_Sample;


#define CONTATORE1_INIZIALE 1000
#define CONTATORE2_INIZIALE 2000

int Encoder_Position;
float Phase_A;
float Phase_B;
float Phase_C;

float Sine_A;
float Sine_B;
float Sine_C;

float ADC_filtrato;

// The program execution by default starts from the function called "main"
int main(void){

    int Rosso=0;
    int Contatore1=0;
    int Contatore2=0;
    int Contatore3=-100;

    int Digit=1;
    int Esito_Display;
    int Number_to_be_Converted;

    int Red_Duty;
    int Blue_Duty;
    int Green_Duty;

    char Four_Digits[4]={'2','1','0','1'};

    char Char_to_Display;


    float Angle;
    float Sine;

    Enable_the_Peripherals();
    Initialize_the_Timer0(10);
    Initialize_the_Clocks();
    Initialize_the_GPIO();
    Initialize_the_ADC();
    Initialize_the_PWM();
    Initialize_the_Encoder();

    Enable_the_Interrupts();

    ATTIVA_IL_TIMER0;

    while (1) {


        while(Contatore1>0){
            Contatore1--;
            while(Contatore2>0){
                Contatore2--;
            }
            Contatore2=CONTATORE2_INIZIALE;


            if(++Digit>4){
                Digit=1;
            }

            Char_to_Display=Four_Digits[Digit-1];

            Esito_Display=Show_Character_on_Digit(Char_to_Display,Digit);

            if(Esito_Display==CORRECT){
                ACCENDI_VERDE;
                SPEGNI_ROSSO;
            }
            else{
                ACCENDI_ROSSO;
                SPEGNI_VERDE;
            }



        }

        Contatore1=CONTATORE1_INIZIALE;

        Encoder_Position=ENCODER_POSITION;

        Phase_A=Encoder_Position*DUE_PI/MAXIMUM_ENCODER_COUNT;
        Phase_B=Phase_A+DUE_PI_3;
        Phase_C=Phase_A-DUE_PI_3;

        Interpolate_Sine_Table(DUE_PI_3, &Sine_A);
        Interpolate_Sine_Table(Phase_B, &Sine_B);
        Interpolate_Sine_Table(Phase_C, &Sine_C);

        Red_Duty=(int)((Sine_A+1)*PWM_TIME_BASE_PERIOD_2);
        Blue_Duty=(int)((Sine_B+1)*PWM_TIME_BASE_PERIOD_2);
        Green_Duty=(int)((Sine_C+1)*PWM_TIME_BASE_PERIOD_2);

        Set_PWM_Duty_Cycles(Red_Duty,Blue_Duty,Green_Duty);

        Number_to_Char_Converter((int)ADC_filtrato,Four_Digits);


        if(Rosso==0){
            Manage_LEDs(TURN_ON,RED);
            Manage_LEDs(TURN_OFF,BLUE);
            Rosso=1;
        }
        else{
            Manage_LEDs(TURN_OFF,RED);
            Manage_LEDs(TURN_ON,BLUE);
            Rosso=0;
        }






    }
}




interrupt void Timer0_isr(){

//    Global_Sample=Sample_the_Analog_Voltage();


    if(BOTTONE_PREMUTO){
//        ACCENDI_ROSSO;
        ACCENDI_BLU;
    }
    else{
//        SPEGNI_ROSSO;
        SPEGNI_BLU;
    }

    RESETTA_IL_TIMER0;
    PieCtrlRegs.PIEACK.bit.ACK1=1;

}


interrupt void ADC_isr(){

#define KY1 0.999371878799377
#define KX 0.314060600311507e-3
#define KX1 0.314060600311507e-3


   float x;
   static float xp=0;
   float y;
   static float yp=0;


   Global_Sample=AdcResult.ADCRESULT3;

   x=(float)Global_Sample;

   y=KY1*yp+KX*x+KX1*xp;

    xp=x;
    yp=y;

    ADC_filtrato=y;

    AdcRegs.ADCINTFLGCLR.bit.ADCINT1=1;
    PieCtrlRegs.PIEACK.bit.ACK1=1;

}





int Number_to_Char_Converter(int Number,char* Char_Vector){
    int Result=CORRECT;
    int Negative=0;

    int Thousands=0;
    int Hundreds=0;
    int Tens=0;
    int Units=0;


    if((Number<=9999) & (Number>=-999)){
        if(Number<0){
            Negative=1;
            Number=-Number;
        }

        Thousands=Number/1000;
        Number=Number-Thousands*1000;

        Hundreds=Number/100;
        Number=Number-Hundreds*100;

        Tens=Number/10;
        Number=Number-Tens*10;

        Units=Number;

        if(Negative==1){
            Char_Vector[0]='-';
        }
        else{
            Char_Vector[0]='0'+Thousands;
        }

        Char_Vector[1]='0'+Hundreds;
        Char_Vector[2]='0'+Tens;
        Char_Vector[3]='0'+Units;


    }
    else{
        Result=ERROR;
        Char_Vector[0]='E';
        Char_Vector[1]='r';
        Char_Vector[2]='r';
        Char_Vector[3]=' ';
    }


    return Result;
}

